global_oil_cols = [
    "Demand",
    "OECD",
    "Non-OECD",
    "Non-OPEC supply",
    "North America",
    "FSU",
    "OPEC NGLs/Condensates",
    "OPEC crude",
    "Stock change",
]

global_oil_basic_summary_cols = [
    "Demand",
    "Non-OPEC supply",
    "OPEC crude",
    "OPEC NGLs/Condensates",
    "Stock change",
]
